<?php

/**
 * creates AssoFacile Admin Menu and encloses AssoFacileSettings object
 * AssoFacileSettings object further registers settings and creates settings controls
 */
class AssoFacileMenu {
  function __construct() {
    // create new admin menu
    add_action('admin_menu', array($this, 'add_admin_menu'), 100);

    // fill up menu page with settings(section, fields)
    new AssoFacileSettings();
  }

  public function add_admin_menu() {
    if (function_exists('add_submenu_page')) {
      add_submenu_page('options-general.php', __('AssoFacile', "assofacile"), __('AssoFacile', "assofacile"), 'manage_options', 'assofacile/configure', array($this, 'admin_menu_page'));
    } else {
      add_menu_page(__('AssoFacile', "assofacile"), __('AssoFacile', "assofacile"), 'manage_options', 'assofacile/configure', array('AssoFacileMenu', 'admin_menu_page'));
    }
  }

  /**
   * html content of menu page
   */
  public function admin_menu_page() {
    ?>
      <h1><?php _e("AssoFacile Plugin", "assofacile") ?></h1>
      <form method="POST" action="options.php">

      <?php
        settings_fields('assofacile');
        do_settings_sections('assofacile/configure');
        submit_button('Save');
      ?>

      </form>
    <?php
  }
}


/**
 * creates Wordpress settings (section and field) for keeping AssoFacile preferences
 */
class AssoFacileSettings {
  private $pages; // all pages
  private $key; // private key
  private $calendar_page; // id of calendar page
  private $reserved_area_page; // id of reserved area page

  function __construct() {
    add_action('admin_init', array($this, 'add_admin_settings'));
  }

  /**
   * register difference settings
   * 1) API Key
   * 2) Calendar Page
   * 3) Reserved Area Page
   */
  public function add_admin_settings() {
    /**
     * Register API Key / Association ID
     */
    register_setting('assofacile', 'assofacile_assoc_id');

    /**
     * Register Api token
     */
    register_setting('assofacile', 'assofacile_api_token');
    register_setting('assofacile', 'assofacile_api_token2');
    register_setting('assofacile', 'token_lgn');

    /**
     * Register allowed tabs
     */
    register_setting('assofacile', 'assofacile_allowed_tabs', [
      "type" => "array"
    ]);

    /**
     * Register Redirect URL
     */
    register_setting('assofacile', 'assofacile_redirect_url');

    /**
     * Register Calendar Page
     */
    register_setting('assofacile', 'assofacile_calendar_page');

    /**
     * Register Reserved Area Page
     */
    register_setting('assofacile', 'assofacile_reserved_area_page');

    /**
     * create section of settings and realted  fields
     */
      add_settings_section('assofacile_configure_section', __('Configurazioni AssoFacile'), array($this, 'cb_add_admin_settings_section_html'), 'assofacile/configure');
      add_settings_field('assofacile_assoc_id', __("ID Associazione"), array($this, "cb_add_assoc_id_field_html"), "assofacile/configure", "assofacile_configure_section");
      add_settings_field('assofacile_calendar_page', __("Pagina calendario"), array($this, "cb_add_calendar_page_filed_html"), "assofacile/configure", "assofacile_configure_section");
      add_settings_field('assofacile_reserved_area_page', __("Pagina area riservata"), array($this, "cb_add_reserved_area_page_field_html"), "assofacile/configure", "assofacile_configure_section");
  }

  /**
   * ouptut of section
   */
  public function cb_add_admin_settings_section_html() {
    ?>
      <p><b><?php _e("This section let you configure API Key and special pages", "assofacile"); ?>!</b></p>
      <p>Potrai usare i seguenti shortcode nelle pagine del tuo sito: <i>[assofacile_calendario]</i> e <i>[assofacile_area_riservata]</i></p>
    <?php
  }

  /**
   * output of adding api key field
   */
  public function cb_add_assoc_id_field_html() {
    $value = get_option('assofacile_assoc_id', '');
    ?>
    <input type="number" name="assofacile_assoc_id" id="assofacile_assoc_id" value="<?php echo $value ?>"
       required>
    <?php
  }

  /**
   * output of adding calendear page field
   */
  public function cb_add_calendar_page_filed_html() {
    $pages = get_pages();
    $value = get_option('assofacile_calendar_page', '');

    ?>
    <select name="assofacile_calendar_page" id="assofacile_calendar_page" value="<?php echo $value ?>">
      <option value=""><?php _e("Please select a page", "assofacile"); ?></option>
      <?php
        foreach ($pages as $page) {
          $id = $page->ID;
          $title = $page->post_title;
          $option = "<option value='$id' ";

          if ($id == $value) {
            $option .= "selected";
          }

          $option .= ">$title</option>";
          echo $option;
        }
      ?>
    </select>
    <?php
  }

  /**
   * output of adding reserved area page field
   */
  public function cb_add_reserved_area_page_field_html() {
    $pages = get_pages();
    $value = get_option('assofacile_reserved_area_page', '');

    ?>
    <select name="assofacile_reserved_area_page" id="assofacile_reserved_area_page" value="<?php echo $value ?>">
      <option value="assofacile_reserved_area_page"><?php _e("Please select a page", "assofacile"); ?></option>
      <?php
        foreach ($pages as $page) {
          $id = $page->ID;
          $title = $page->post_title;
          $option = "<option value='$id' ";

          if ($id == $value) {
            $option .= "selected";
          }

          $option .= ">$title</option>";
          echo $option;
        }
      ?>
    </select>
    <?php
  }
}

new AssoFacileMenu();

?>