/**
 */
function fetchProgress(url, fetchOptions, responseType, progress) {
  return fetch(url, fetchOptions)
    .then(response => {
      if (!response.ok) {
        throw Error(response.status+' '+response.statusText)
      }

      if (!response.body) {
        throw Error('ReadableStream not yet supported in this browser.')
      }

      let contentLength;
      try {
        contentLength = response.headers.get('x-file-size');
      } catch {
        try {
          contentLength = response.headers.get('content-length');
        } catch {
          contentLength = 0;
        }
      }

      const total = parseInt(contentLength, 10);
      let loaded = 0;

      return new Response(
        new ReadableStream({
          start(controller) {
            const reader = response.body.getReader();

            read();
            function read() {
              reader.read().then(({done, value}) => {
                if (done) {
                  controller.close();
                  return; 
                }

                loaded += value.byteLength;

                let percentage;

                if (loaded && loaded < total)
                  percentage = (((loaded * 100) / total) % 100).toPrecision(3);
                else
                  percentage = 100;

                if (progress)
                  progress({loaded, total, percentage});

                controller.enqueue(value);
                read();
              }).catch(error => {
                console.error(error);
                controller.error(error)                  
              })
            }
          }
        })
      , {
        headers: response.headers 
      });
    }).then((response) => {
      if (responseType == 'blob') {
        return response.blob();
      } else if (responseType == 'text') {
        return response.text();
      } else if (responseType == 'json') {
        return response.json();
      }

      return response;
    })
    .catch(error => {
      throw error;
    });
}

if (!window.toFixed) {
  window.toFixed = (num, precision) => (+(Math.round(+(num + 'e' + precision)) + 'e' + -precision)).toFixed(precision);
}

// base64 to buffer
function base64ToBufferAsync(base64) {
  const dataUrl = "data:application/octet-binary;base64," + base64;
  return fetch(dataUrl).then(res => res.arrayBuffer());
}