/**
 * create an input (or input like) elemement form given object
 */
class ControlFactory {
  constructor(properties) {
    this.$ = jQuery;

    const container = this.$(document.createElement('div'))
      .addClass('form-group');

    if (properties['label'] || properties['title']) {
      container.append(
        this.$(document.createElement('label'))
          .attr('for', properties['name'])
          .html((properties['label'] || properties['title']) +
            (properties['required'] ? '<span class="text-danger" style="padding-left: 4px">*</span>' : ''))
      );
    }

    if (properties['type'] == 'array') {
      container.append(
        this.$(document.createElement('label'))
          .attr('for', properties['name'])
          .html(properties['name'].toUpperCase() +
            (properties['required'] ? '<span class="text-danger" style="padding-left: 4px">*</span>' : ''))
      );
    }

    if (properties['description']) {
      container.append(
        this.$(document.createElement('p'))
          .append(
            this.$(document.createElement('small'))
              .text(properties['description']
          )
        )
      );
    }

    if (properties['file']) {
      try {
        const link = this.$(document.createElement("a")).attr("target", "_blank");

        function _base64ToArrayBuffer(base64) {
          var binary_string = window.atob(base64);
          var len = binary_string.length;
          var bytes = new Uint8Array(len);
          for (var i = 0; i < len; i++) {
            bytes[i] = binary_string.charCodeAt(i);
          }
          return bytes.buffer;
        }

        const file = new File([_base64ToArrayBuffer(properties.file.data)], properties.file.filename, {
          type: properties.file["content-type"],
        });

        const url = URL.createObjectURL(file);
        link.attr("href", url);


        link.text('Download File');
        container.append(
          this.$(document.createElement('div'))
            .append(link)
        );
      } catch (err) {
        console.log(err);
      }
    }


    let control;
    const type = (properties['type'] || properties['@type'] || '').toLowerCase();
    switch (type) {
      case 'text':
        control = this.$(document.createElement('input'))
          .addClass('form-control')
          .attr('type', 'text');
        break;

      case 'hidden':
        control = this.$(document.createElement('input'))
          .addClass('form-control')
          .attr('type', 'hidden');
        break;

      case 'radio':
        control = this.getRadioOptionsGroup(properties);
        break;

      case 'file':
        control = this.$(document.createElement('input'))
          .addClass('form-control')
          .attr('type', 'file');
        break;

      case 'textarea':
        control = this.$(document.createElement('textarea'))
          .addClass('form-control');
        break;

      case 'select':
        control = this.$(document.createElement('select'))
          .addClass('form-control')
          .append(this.getOptions(properties, 'options'));
          break;

      case 'array':
        control = this.$(document.createElement('select'))
          .addClass('form-control')
          .append(this.getOptions(properties, 'values'));
          break
        
      default:
        control = {};
    }

    if (properties['required'])
      control.attr('required', true);

    this.setAttributes(properties, control);
    container.append(control);
    this.element = container;
  }

  setAttributes(properties, control) {
    /**
     * skip `type`, '@type', `label`, required, `description` and options properties
     */
    for (const prop in properties) {
      if (!properties.hasOwnProperty(prop) 
        || prop == 'type' 
        || prop == '@type' 
        || prop == 'label'
        || prop == 'required'
        || prop == 'options'
        || prop == 'description'
      ) {
        continue;
      }

      control.attr(prop, properties[prop]);
    }
  }

  /**
   * do not call from outsize of class
   */
  getOptions(properties, optionsPropName) {
    let options = [];
    for (const option of properties[optionsPropName]) {
      options.push(this.$(document.createElement('option'))
        .attr('value', option['value'])
        .attr('id', option['id'])
        .text(option['value'])
      );
    }

    return options;
  }

  /**
   * do not call from outsize of class
   */
  getRadioOptionsGroup(properties) {
    const group = this.$(document.createElement('div'))
      .addClass('d-flex flex-row flex-wrap p-1');

      for (const option of properties['options']) {
        group.append(this.$(document.createElement('div'))
          .addClass('input-group')
          .append(
            this.$(document.createElement('input'))
              .attr('type', 'radio')
              .attr('value', option['value'])
              .attr('name', properties['name'])
              .attr('id', option['id']),

            this.$(document.createElement('span'))
              .attr('style', 'padding: 0px 10px;')
              .html(option['text'])
          )
        );
      }

    return group;
  }

  getElement() {
    return this.element;
  }
}

globalThis['ControlFactory'] = ControlFactory;